/*==============================================================================
    FILE:           user_hid.c

    DESCRIPTION:    Uses the cld_hid library to implement a basic USB mouse
                    HID device.

                    This example uses the ADSP-BF707 EZ-Board push buttons to
                    generate HID Mouse reports.

                    Push Button 2 selects which mouse event to generate
                    (Button Press, Move Up, Move Down, Move Left, or Move Right).
                    Push Button 1 creates the selected mouse event.


    Copyright (c) 2015 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to reproduce and distribute executable files
    created using this software for use on Analog Devices Blackfin family
    processors only. Nothing else gives you the right to use this software.

==============================================================================*/
#include "user_hid.h"
#include "cld_bf70x_hid_lib.h"

#include <ADSP-BF707_device.h>

/* Mouse HID Report Descriptor */
static const unsigned char user_hid_report_descriptor[] =
{
    0x05, 0x01,                    /* USAGE_PAGE (Generic Desktop)      */
    0x09, 0x02,                    /* USAGE (Mouse)                     */
    0xa1, 0x01,                    /* COLLECTION (Application)          */
    0x09, 0x01,                    /*   USAGE (Pointer)                 */
    0xa1, 0x00,                    /*   COLLECTION (Physical)           */
    0x05, 0x09,                    /*     USAGE_PAGE (Button)           */
    0x19, 0x01,                    /*     USAGE_MINIMUM (Button 1)      */
    0x29, 0x03,                    /*     USAGE_MAXIMUM (Button 3)      */
    0x15, 0x00,                    /*     LOGICAL_MINIMUM (0)           */
    0x25, 0x01,                    /*     LOGICAL_MAXIMUM (1)           */
    0x95, 0x03,                    /*     REPORT_COUNT (3)              */
    0x75, 0x01,                    /*     REPORT_SIZE (1)               */
    0x81, 0x02,                    /*     INPUT (Data,Var,Abs)          */
    0x95, 0x01,                    /*     REPORT_COUNT (1)              */
    0x75, 0x05,                    /*     REPORT_SIZE (5)               */
    0x81, 0x03,                    /*     INPUT (Cnst,Var,Abs)          */
    0x05, 0x01,                    /*     USAGE_PAGE (Generic Desktop)  */
    0x09, 0x30,                    /*     USAGE (X)                     */
    0x09, 0x31,                    /*     USAGE (Y)                     */
    0x15, 0x81,                    /*     LOGICAL_MINIMUM (-127)        */
    0x25, 0x7f,                    /*     LOGICAL_MAXIMUM (127)         */
    0x75, 0x08,                    /*     REPORT_SIZE (8)               */
    0x95, 0x02,                    /*     REPORT_COUNT (2)              */
    0x81, 0x06,                    /*     INPUT (Data,Var,Rel)          */
    0xc0,                          /*   END_COLLECTION                  */
    0xc0                           /* END_COLLECTION                    */
};

/* Function Prototypes. */
static void user_hid_usb_in_transfer_complete (void);
static CLD_USB_Transfer_Request_Return_Type user_hid_set_report_received (unsigned char report_id, CLD_USB_Transfer_Params * p_transfer_data);
static CLD_USB_Transfer_Request_Return_Type user_hid_get_report_received (unsigned char report_id, CLD_USB_Transfer_Params * p_transfer_data);
static void user_hid_console_rx_byte (unsigned char byte);
static void user_hid_usb_event_callback (CLD_USB_Event event);

static void user_hid_monitor_push_buttons (void);
static void user_hid_push_button_mode_pressed (void);
static void user_hid_push_button_activity_pressed (void);
static void user_hid_push_button_activity_released (void);
static void user_hid_activity_pressed_mouse_click (void);
static void user_hid_activity_released_mouse_click (void);
static void user_hid_activity_pressed_mouse_up (void);
static void user_hid_activity_released_mouse_up (void);
static void user_hid_activity_pressed_mouse_down (void);
static void user_hid_activity_released_mouse_down (void);
static void user_hid_activity_pressed_mouse_left (void);
static void user_hid_activity_released_mouse_left (void);
static void user_hid_activity_pressed_mouse_right (void);
static void user_hid_activity_released_mouse_right (void);

#define USER_HID_NUM_ACTIVITIES     5
/* Functions called when the PB1 button is pressed.
   This function table indexed using user_hid_current_activity. */
static void (* const user_hid_activity_pressed_function_tbl[USER_HID_NUM_ACTIVITIES]) (void) =
{
    user_hid_activity_pressed_mouse_click,
    user_hid_activity_pressed_mouse_up,
    user_hid_activity_pressed_mouse_down,
    user_hid_activity_pressed_mouse_left,
    user_hid_activity_pressed_mouse_right,
};

/* Functions called when the PB1 button is Released.
   This function table indexed using user_hid_current_activity. */
static void (* const user_hid_activity_released_function_tbl[USER_HID_NUM_ACTIVITIES]) (void) =
{
    user_hid_activity_released_mouse_click,
    user_hid_activity_released_mouse_up,
    user_hid_activity_released_mouse_down,
    user_hid_activity_released_mouse_left,
    user_hid_activity_released_mouse_right,
};

/* Array used to output which mouse mode is currently active.
   This array indexed using user_hid_current_activity. */
static char * user_hid_mode_string[USER_HID_NUM_ACTIVITIES] =
{
    "Button Click",
    "Move Up",
    "Move Down",
    "Move Left",
    "Move Right",
};

static unsigned char user_hid_current_activity = 0;

/* Required HID Interrupt IN endpoint parameters. */
static CLD_HID_Endpoint_Params user_hid_interrupt_in_params =
{
    .endpoint_number                = 1,
    .max_packet_size_full_speed     = 64,
    .polling_interval_full_speed    = 1,
    .max_packet_size_high_speed     = 64,
    .polling_interval_high_speed    = 4,  /* 1ms */
};

/* cld_hid library initialization data. */
CLD_BF70x_HID_Lib_Init_Params user_hid_init_params =
{
    .timer_num  = CLD_TIMER_0,                          /* Timer used by the CLD Library */
    .uart_num   = CLD_UART_0,                           /* UART used by the CLD Library.
                                                            If the uart_num is set to
                                                            CLD_UART_DISABLE the CLD library
                                                            will not use a UART */
    .uart_baud  = 115200,                               /* CLD Library CONSOLE print UART
                                                           baudrate. */
    .sclk0      = 100000000u,                           /* Blackfin SCLK0 frequency */
    .fp_console_rx_byte = user_hid_console_rx_byte,     /* Function called when a byte
                                                           is received by the CONSOLE
                                                           UART. */
    .vendor_id = 0x064b,                                /* Analog Devices Vendor ID */
    .product_id = 0x0001,                               /* Product ID supplied by DSP Concepts */

    .report_descriptor_size = sizeof(user_hid_report_descriptor),
    .p_report_descriptor = (unsigned char *)user_hid_report_descriptor,

    /* HID Interrupt IN parameters */
    .p_interrupt_in_endpoint_params = &user_hid_interrupt_in_params,

    /* Optional HID Interrupt OUT endpoint parameters. If the Interrupt OUT
       endpoint is not used these pointers should be set to CLD_NULL. */
    .p_interrupt_out_endpoint_params = CLD_NULL,
    .fp_interrupt_out_data_received = CLD_NULL,

    .usb_bus_max_power = 0,                             /* The ADSP-BF707 EZ-Board is
                                                           self-powered (not USB bus-powered) */

    .device_descriptor_bcdDevice = 0x0100,              /* Set USB Device Descriptor
                                                           firmware version to 1.00 */
    /* USB string descriptors - Set to CLD_NULL if not required */
    .p_usb_string_manufacturer  = "Closed Loop Design, LLC",
    .p_usb_string_product       = "BF707 USB Mouse",
    .p_usb_string_serial_number = "00000001",
    .p_usb_string_configuration = CLD_NULL,
    .p_usb_string_interface     = "CLD HID Mouse",
    .usb_string_language_id     = 0x0409,               /* English (US) language ID */

     /* Function called when a HID Set Report Command is received. */
    .fp_set_report_received = user_hid_set_report_received,
     /* Function called when a HID Get Report Command is received. */
    .fp_get_report_received = user_hid_get_report_received,
     /* Function called when a HID Set Idle Command is received. Set to CLD_NULL
        if the device doesn't support the Set Idle Command */
    .fp_get_idle = CLD_NULL,
     /* Function called when a HID Get Idle Command is received. Set to CLD_NULL
        if the device doesn't support the Get Idle Command */
    .fp_set_idle = CLD_NULL,

    /* Function called when a USB events occurs. */
    .fp_cld_usb_event_callback = user_hid_usb_event_callback,
};

/* USB HID Mouse data structure. */
#pragma pack (1)
typedef struct
{
    unsigned char button;   /* Mouse button state  */
    signed char x;          /* X position */
    signed char y;          /* Y position */
} User_HID_Mouse_Input_Report;
#pragma pack ()

/* Mouse input Report. */
section("sdram0")static User_HID_Mouse_Input_Report user_hid_mouse_input_report;

/* CLD Transfer params used to report the mouse position data using the Interrupt
   IN endpoint. */
static CLD_USB_Transfer_Params user_hid_mouse_input_report_transfer_params =
{
    .num_bytes = sizeof(user_hid_mouse_input_report),
    .p_data_buffer = (unsigned char *)&user_hid_mouse_input_report,
    .callback.fp_usb_in_transfer_complete = user_hid_usb_in_transfer_complete,
    .fp_transfer_aborted_callback = CLD_NULL,
    .transfer_timeout_ms = 1000,
};


/*=============================================================================
Function:       user_hid_init

Parameters:     None.

Description:    Initializes the CLD HID library.

Returns:        USER_HID_INIT_SUCCESS/USER_HID_INIT_ONGOING/USER_HID_INIT_FAILED.
==============================================================================*/
User_HID_Init_Return_Code user_hid_init (void)
{
    static unsigned char user_init_state = 0;
    CLD_RV cld_rv = CLD_ONGOING;

    /* Initialize the CLD HID Library */
    cld_rv = cld_bf70x_hid_lib_init(&user_hid_init_params);

    if (cld_rv == CLD_SUCCESS)
    {
        /* Connect to the USB Host */
        cld_lib_usb_connect();

        return USER_HID_INIT_SUCCESS;
    }
    else if (cld_rv == CLD_FAIL)
    {
        return USER_HID_INIT_FAILED;
    }
    else
    {
        return USER_HID_INIT_ONGOING;
    }
}

/*=============================================================================
Function:       user_hid_main

Parameters:     None.

Description:    Example user mainline.

Returns:        None.
==============================================================================*/
void user_hid_main (void)
{
    #define MINUTES(x)      (x/60000)
    #define SECONDS(x)      ((x/1000)%60)
    #define M_SECONDS(x)    (x%1000)

    static CLD_Time main_time = 0;
    static CLD_Time msg_time = 0;
    static CLD_Time run_time = 0;

    cld_bf70x_hid_lib_main();

    if (cld_time_passed_ms(main_time) >= 250u)
    {
        main_time = cld_time_get();
        pADI_PORTA->DATA_TGL = (1 << 0);
    }
    if (cld_time_passed_ms(msg_time) >= 237u)
    {
        msg_time = cld_time_get();
        run_time = cld_time_passed_ms(0);
        cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "Run time: %2.2d:%2.2d:%3.3d\r",
            MINUTES(run_time),
            SECONDS(run_time),
            M_SECONDS(run_time));
    }

    user_hid_monitor_push_buttons();
}

/*=============================================================================
Function:       user_hid_usb_in_transfer_complete

Parameters:     None.

Description:    This function is called by the cld_hid library when the Get Report
                data has been sent to the Host.

Returns:        None.
==============================================================================*/
static void user_hid_usb_in_transfer_complete (void)
{
    /* The Get Report data has been send to the Host. */
    pADI_PORTB->DATA_TGL = (1 << 1);
}

/*=============================================================================
Function:       user_hid_set_report_received

Parameters:     p_transfer_data - Pointer to the Control OUT transfer data.
                    p_transfer_data->num_bytes     - Number of Control OUT bytes.
                    p_transfer_data->p_data_buffer - Set to the address where to
                                                     store the Control OUT data.
                    p_transfer_data->callback.usb_in_transfer_complete -
                                                     Function called when the
                                                     Control OUT bytes have been
                                                     received.
                    p_transfer_data->transfer_aborted_callback -
                                                     Optional function that is
                                                     called if the transfer is
                                                     aborted.

Description:    This function is called by the cld_hid library the HID Set
                Report command is received. This function sets the
                p_transfer_data parameters to select where the received data
                should be stored, and what function should be called when the
                transfer is complete.

Returns:        CLD_USB_TRANSFER_ACCEPT - Store the data using the p_transfer_data
                                          parameters.
                CLD_USB_TRANSFER_PAUSE - The device isn't ready to process this
                                         Control OUT request so pause the transfer
                                         until the cld_hid_resume_paused_control_transfer
                                         function is called.
                CLD_USB_TRANSFER_DISCARD - Discard the received data.
                CLD_USB_TRANSFER_STALL - Stall the Control endpoint.
==============================================================================*/
static CLD_USB_Transfer_Request_Return_Type user_hid_set_report_received (unsigned char report_id, CLD_USB_Transfer_Params * p_transfer_data)
{
    /* The HID Mouse does not include and HID OUTPUT or FEATURE Reports
       so Set Report is not used. */
    return CLD_USB_TRANSFER_STALL;
}

/*=============================================================================
Function:       user_hid_get_report_received

Parameters:     p_transfer_data - Pointer to the Control IN transfer data.
                    p_transfer_data->num_bytes     - Number of Control IN bytes.
                    p_transfer_data->p_data_buffer - Set to the address where to
                                                     source the Control IN data.
                    p_transfer_data->callback.usb_in_transfer_complete -
                                                     Function called when the
                                                     Control IN bytes have been
                                                     sent to the Host.
                    p_transfer_data->transfer_aborted_callback -
                                                     Optional function that is
                                                     called if the transfer is
                                                     aborted.

Description:    This function is called by the cld_hid library a HID Get Report
                command is received. This function sets the p_transfer_data
                parameters to return the requested data to the Host,
                and what function should be called when the transfer is complete.

Returns:        CLD_USB_TRANSFER_ACCEPT - Transmit the specified data to the Host.
                CLD_USB_TRANSFER_PAUSE - The device isn't ready to process this
                                         Control IN request so pause the transfer
                                         until the cld_hid_resume_paused_control_transfer
                                         function is called.
                CLD_USB_TRANSFER_DISCARD - Return a zero length packet to the Host.
                CLD_USB_TRANSFER_STALL - Stall the Control endpoint.
==============================================================================*/
static CLD_USB_Transfer_Request_Return_Type user_hid_get_report_received (unsigned char report_id, CLD_USB_Transfer_Params * p_transfer_data)
{
    /* Set p_transfer_data to return the current position and button
       status to the Host, and call user_hid_usb_in_transfer_complete when
       done. */
    p_transfer_data->num_bytes = sizeof(user_hid_mouse_input_report);
    p_transfer_data->p_data_buffer = (unsigned char *)&user_hid_mouse_input_report;
    p_transfer_data->callback.fp_usb_in_transfer_complete = user_hid_usb_in_transfer_complete;
    p_transfer_data->fp_transfer_aborted_callback = CLD_NULL;
    p_transfer_data->transfer_timeout_ms = 0;

    return CLD_USB_TRANSFER_ACCEPT;
}


/*=============================================================================
Function:       user_hid_console_rx_byte

Parameters:     byte - received byte.

Description:    Function called when the cld_console UART receives a byte.

Returns:        None.
==============================================================================*/
static void user_hid_console_rx_byte (unsigned char byte)
{
    static const CLD_CONSOLE_COLOR cld_console_colors[CLD_CONSOLE_NUM_COLORS] =
    {
        CLD_CONSOLE_RED,
        CLD_CONSOLE_GREEN,
        CLD_CONSOLE_YELLOW,
        CLD_CONSOLE_BLUE,
        CLD_CONSOLE_PURPLE,
        CLD_CONSOLE_CYAN,
        CLD_CONSOLE_WHITE,
        CLD_CONSOLE_BLACK,
    };
    static const char * color_strings[CLD_CONSOLE_NUM_COLORS] =
    {
        "Red",
        "Green",
        "Yellow",
        "Blue",
        "Purple",
        "Cyan",
        "White",
        "Black",
    };
    unsigned char i;

    cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "\r\nUser received %c\r\n", byte);

    if (byte == '!')
    {
        cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "\n\r----- CLD Console Color Demo -----\n\r\n\r");
        for (i = 0; i < CLD_CONSOLE_NUM_COLORS; i++)
        {
            cld_console(cld_console_colors[i], cld_console_colors[CLD_CONSOLE_NUM_COLORS - i - 1], "%s text with a %s background\n\r",
                        color_strings[i], color_strings[CLD_CONSOLE_NUM_COLORS - i - 1]);
        }
        cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "\n\r----------------------------------\n\r\n\r");
    }
}

/*=============================================================================
Function:       user_hid_usb_event_callback

Parameters:     event - identifies which USB event has occurred.

Description:    Function Called when a USB event occurs.

Returns:        None.
==============================================================================*/
static void user_hid_usb_event_callback (CLD_USB_Event event)
{
    switch (event)
    {
        case CLD_USB_CABLE_CONNECTED:
            pADI_PORTA->DATA_SET = (1 << 1);
        break;
        case CLD_USB_CABLE_DISCONNECTED:
            pADI_PORTA->DATA_CLR = (1 << 1);
        break;
        case CLD_USB_ENUMERATED_CONFIGURED:
            cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "CLD HID Mouse Enumerated\n\r");
        break;
        case CLD_USB_UN_CONFIGURED:
        break;
        case CLD_USB_BUS_RESET:
        break;
        case CLD_USB_BUS_SUSPEND:
            cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "CLD HID Device Suspend\n\r");
        break;
        case CLD_USB_BUS_RESUME:
            cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "CLD HID Device Resume\n\r");
        break;
    }
}

/*=============================================================================
Function:       user_hid_monitor_push_buttons

Parameters:     None.

Description:    Monitors the ADSP-BF707 EZ-Board push button inputs,

Returns:        None.
==============================================================================*/
static void user_hid_monitor_push_buttons (void)
{
    /* PA.2 PB 1
       PB.7 PB 2 */
    #define USER_HID_PUSH_BUTTON_MODE   (1 << 7)
    #define USER_HID_PUSH_BUTTON_ACTIVITY (1 << 2)
    static unsigned char prev_mode_button = 0;
    static unsigned char prev_activity_button = 0;

    /* If the Mode push button (Port B pin 7) was pressed. */
    if ((pADI_PORTB->DATA & USER_HID_PUSH_BUTTON_MODE) && (prev_mode_button == 0))
    {
        prev_mode_button = 1;
        user_hid_push_button_mode_pressed();
    }
    /* If the Mode push button (Port B pin 7) was released. */
    else if (((pADI_PORTB->DATA & USER_HID_PUSH_BUTTON_MODE) == 0) && (prev_mode_button == 1))
    {
        prev_mode_button = 0;
    }

    /* If the Activity push button (Port A pin 2) was pressed. */
    if ((pADI_PORTA->DATA & USER_HID_PUSH_BUTTON_ACTIVITY) && (prev_activity_button == 0))
    {
        prev_activity_button = 1;
        user_hid_push_button_activity_pressed();
    }
    /* If the Activity push button (Port A pin 2) was released. */
    else if (((pADI_PORTA->DATA & USER_HID_PUSH_BUTTON_ACTIVITY) == 0) && (prev_activity_button == 1))
    {
        prev_activity_button = 0;
        user_hid_push_button_activity_released();
    }
}

/*=============================================================================
Function:       user_hid_push_button_mode_pressed

Parameters:     None.

Description:    Function called when the mode push button is pressed.

Returns:        None.
==============================================================================*/
static void user_hid_push_button_mode_pressed (void)
{
    /* Call the released activity function for the current activity. */
    if (user_hid_activity_released_function_tbl[user_hid_current_activity] != CLD_NULL)
    {
        user_hid_activity_released_function_tbl[user_hid_current_activity]();
    }

    /* Cycle through the available Mouse event modes and output the current
       mode using the cld_console. */
    user_hid_current_activity++;
    if (user_hid_current_activity >= USER_HID_NUM_ACTIVITIES)
    {
        user_hid_current_activity = 0;
    }
    cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK,"HID Mouse Mode: %s\n\r", user_hid_mode_string[user_hid_current_activity]);
}

/*=============================================================================
Function:       user_hid_push_button_activity_pressed

Parameters:     None.

Description:    Function called when the activity push button is pressed.

Returns:        None.
==============================================================================*/
static void user_hid_push_button_activity_pressed (void)
{
    if (user_hid_activity_pressed_function_tbl[user_hid_current_activity] != CLD_NULL)
    {
        user_hid_activity_pressed_function_tbl[user_hid_current_activity]();
    }
}

/*=============================================================================
Function:       user_hid_push_button_activity_released

Parameters:     None.

Description:    Function called when the activity push button is released.

Returns:        None.
==============================================================================*/
static void user_hid_push_button_activity_released (void)
{
    if (user_hid_activity_released_function_tbl[user_hid_current_activity] != CLD_NULL)
    {
        user_hid_activity_released_function_tbl[user_hid_current_activity]();
    }
}

/*=============================================================================
Function:       user_hid_push_button_activity_pressed

Parameters:     None.

Description:    Function called when the activity push button is pressed and
                the push button mode is active.

Returns:        None.
==============================================================================*/
static void user_hid_activity_pressed_mouse_click (void)
{
    if (user_hid_mouse_input_report.button == 0)
    {
        user_hid_mouse_input_report.button = 0x01;
        cld_bf70x_hid_lib_transmit_interrupt_in_data(&user_hid_mouse_input_report_transfer_params);
    }
}

/*=============================================================================
Function:       user_hid_activity_released_mouse_click

Parameters:     None.

Description:    Function called when the activity push button is released and
                the push button mode is active.

Returns:        None.
==============================================================================*/
static void user_hid_activity_released_mouse_click (void)
{
    if (user_hid_mouse_input_report.button == 0x01)
    {
        user_hid_mouse_input_report.button = 0x00;
        cld_bf70x_hid_lib_transmit_interrupt_in_data(&user_hid_mouse_input_report_transfer_params);
    }
}


/*=============================================================================
Function:       user_hid_activity_pressed_mouse_up

Parameters:     None.

Description:    Function called when the activity push button is pressed and
                the Move Up mode is active.

Returns:        None.
==============================================================================*/
static void user_hid_activity_pressed_mouse_up (void)
{
    if (user_hid_mouse_input_report.y == 0)
    {
        user_hid_mouse_input_report.y = -64;
        cld_bf70x_hid_lib_transmit_interrupt_in_data(&user_hid_mouse_input_report_transfer_params);
    }
}

/*=============================================================================
Function:       user_hid_activity_released_mouse_up

Parameters:     None.

Description:    Function called when the activity push button is released and
                the Move Up mode is active.

Returns:        None.
==============================================================================*/
static void user_hid_activity_released_mouse_up (void)
{
    if (user_hid_mouse_input_report.y == -64)
    {
        user_hid_mouse_input_report.y = 0x00;
        cld_bf70x_hid_lib_transmit_interrupt_in_data(&user_hid_mouse_input_report_transfer_params);
    }
}

/*=============================================================================
Function:       user_hid_activity_pressed_mouse_down

Parameters:     None.

Description:    Function called when the activity push button is pressed and
                the Move Down mode is active.

Returns:        None.
==============================================================================*/
static void user_hid_activity_pressed_mouse_down (void)
{
    if (user_hid_mouse_input_report.y == 0)
    {
        user_hid_mouse_input_report.y = 64;
        cld_bf70x_hid_lib_transmit_interrupt_in_data(&user_hid_mouse_input_report_transfer_params);
    }
}

/*=============================================================================
Function:       user_hid_activity_released_mouse_down

Parameters:     None.

Description:    Function called when the activity push button is released and
                the Move Down mode is active.

Returns:        None.
==============================================================================*/
static void user_hid_activity_released_mouse_down (void)
{
    if (user_hid_mouse_input_report.y == 64)
    {
        user_hid_mouse_input_report.y = 0x00;
        cld_bf70x_hid_lib_transmit_interrupt_in_data(&user_hid_mouse_input_report_transfer_params);
    }
}

/*=============================================================================
Function:       user_hid_activity_pressed_mouse_left

Parameters:     None.

Description:    Function called when the activity push button is pressed and
                the Move Left mode is active.

Returns:        None.
==============================================================================*/
static void user_hid_activity_pressed_mouse_left (void)
{
    if (user_hid_mouse_input_report.x == 0)
    {
        user_hid_mouse_input_report.x = -64;
        cld_bf70x_hid_lib_transmit_interrupt_in_data(&user_hid_mouse_input_report_transfer_params);
    }
}

/*=============================================================================
Function:       user_hid_activity_released_mouse_left

Parameters:     None.

Description:    Function called when the activity push button is released and
                the Move Left mode is active.

Returns:        None.
==============================================================================*/
static void user_hid_activity_released_mouse_left (void)
{
    if (user_hid_mouse_input_report.x == -64)
    {
        user_hid_mouse_input_report.x = 0x00;
        cld_bf70x_hid_lib_transmit_interrupt_in_data(&user_hid_mouse_input_report_transfer_params);
    }
}

/*=============================================================================
Function:       user_hid_activity_pressed_mouse_right

Parameters:     None.

Description:    Function called when the activity push button is pressed and
                the Move Right mode is active.

Returns:        None.
==============================================================================*/
static void user_hid_activity_pressed_mouse_right (void)
{
    if (user_hid_mouse_input_report.x == 0)
    {
        user_hid_mouse_input_report.x = 64;
        cld_bf70x_hid_lib_transmit_interrupt_in_data(&user_hid_mouse_input_report_transfer_params);
    }
}

/*=============================================================================
Function:       user_hid_activity_released_mouse_right

Parameters:     None.

Description:    Function called when the activity push button is released and
                the Move Right mode is active.

Returns:        None.
==============================================================================*/
static void user_hid_activity_released_mouse_right (void)
{
    if (user_hid_mouse_input_report.x == 64)
    {
        user_hid_mouse_input_report.x = 0x00;
        cld_bf70x_hid_lib_transmit_interrupt_in_data(&user_hid_mouse_input_report_transfer_params);
    }
}
